import sys
from distutils.version import LooseVersion

from django.urls import clear_url_caches

import cms
from cms.api import add_plugin, create_page
from cms.appresolver import clear_app_resolvers
from cms.test_utils.testcases import CMSTestCase


# These means "less than or equal"
CMS_3_6 = LooseVersion(cms.__version__) < LooseVersion('4.0')


class SubmitFormViewTest(CMSTestCase):

    def setUp(self):
        self.APP_MODULE = 'aldryn_forms.cms_apps.FormsApp'
        clear_app_resolvers()
        clear_url_caches()

        if self.APP_MODULE in sys.modules:
            del sys.modules[self.APP_MODULE]

        self.page = create_page(
            'tpage',
            'test_page.html',
            'en',
            published=True,
            apphook='FormsApp',
        )
        try:
            self.placeholder = self.page.placeholders.get(slot='content')
        except AttributeError:
            self.placeholder = self.page.get_placeholders('en').get(slot='content')

        self.redirect_url = 'http://www.google.com'

        plugin_data = {
            'redirect_type': 'redirect_to_url',
            'url': self.redirect_url,
        }
        self.form_plugin = add_plugin(self.placeholder, 'FormPlugin', 'en', **plugin_data)  # noqa: E501

        add_plugin(
            self.placeholder,
            'SubmitButton',
            'en',
            target=self.form_plugin,
            label='Submit',
        )
        self.form_plugin.action_backend = 'default'
        self.form_plugin.save()
        if CMS_3_6:
            self.page.publish('en')

        self.reload_urls()
        self.apphook_clear()

    def tearDown(self):
        clear_app_resolvers()
        clear_url_caches()

        if self.APP_MODULE in sys.modules:
            del sys.modules[self.APP_MODULE]

        self.reload_urls()
        self.apphook_clear()

    def reload_urls(self):
        from django.conf import settings

        url_modules = [
            'cms.urls',
            self.APP_MODULE,
            settings.ROOT_URLCONF,
        ]

        clear_app_resolvers()
        clear_url_caches()

        for module in url_modules:
            if module in sys.modules:
                del sys.modules[module]

    def test_form_view_and_submission_with_apphook_django_gte_111(self):
        if CMS_3_6:
            public_page = self.page.publisher_public
        else:
            public_page = self.page
        try:
            public_placeholder = public_page.placeholders.first()
        except AttributeError:
            public_placeholder = public_page.get_placeholders('en').first()

        public_page_form_plugin = (
            public_placeholder
            .cmsplugin_set
            .filter(plugin_type='FormPlugin')
            .first()
        )
        response = self.client.get(self.page.get_absolute_url('en'))

        input_string = '<input type="hidden" name="form_plugin_id" value="{}"'
        self.assertContains(response, input_string.format(public_page_form_plugin.id))  # noqa: E501

        response = self.client.post(self.page.get_absolute_url('en'), {
            'form_plugin_id': public_page_form_plugin.id,
        })
        self.assertRedirects(response, self.redirect_url, fetch_redirect_response=False)  # noqa: E501

    def test_view_submit_one_form_instead_multiple(self):
        """Test checks if only one form is send instead of multiple on page together"""
        page = create_page(
            'multiple forms',
            'test_page.html',
            'en',
            published=True,
            apphook='FormsApp',
        )
        placeholder = page.placeholders.get(slot='content')

        form_plugin = add_plugin(
            placeholder,
            'FormPlugin',
            'en',
        )  # noqa: E501

        add_plugin(
            placeholder,
            'EmailField',
            'en',
            name='email_1',
            required=True,
            target=form_plugin,
            label='Submit',
        )

        add_plugin(
            placeholder,
            'SubmitButton',
            'en',
            target=form_plugin,
            label='Submit',
        )

        form_plugin.action_backend = 'default'
        form_plugin.save()

        plugin_data2 = {
            'redirect_type': 'redirect_to_url',
            'url': 'https://google.com/',
        }

        form_plugin2 = add_plugin(
            placeholder,
            'FormPlugin',
            'en',
            **plugin_data2
        )  # noqa: E501

        add_plugin(
            placeholder,
            'SubmitButton',
            'en',
            target=form_plugin2,
            label='Submit',
        )

        form_plugin2.action_backend = 'default'
        form_plugin2.save()

        page.publish('en')
        self.reload_urls()
        self.apphook_clear()

        response = self.client.post(page.get_absolute_url('en'), {
            'form_plugin_id': form_plugin2.id,
            'email_1': 'test@test',
        })
        self.assertRedirects(response, plugin_data2['url'], fetch_redirect_response=False)  # noqa: E501

    def test_view_submit_one_valid_form_instead_multiple(self):
        """Test checks if only one form is validated instead multiple on a page"""
        page = create_page(
            'multiple forms',
            'test_page.html',
            'en',
            published=True,
            apphook='FormsApp',
        )
        placeholder = page.placeholders.get(slot='content')

        form_plugin = add_plugin(
            placeholder,
            'FormPlugin',
            'en',
        )  # noqa: E501

        add_plugin(
            placeholder,
            'EmailField',
            'en',
            name='email_1',
            required=True,
            target=form_plugin,
        )

        add_plugin(
            placeholder,
            'SubmitButton',
            'en',
            target=form_plugin,
            label='Submit',
        )

        form_plugin.action_backend = 'default'
        form_plugin.save()

        form_plugin2 = add_plugin(
            placeholder,
            'FormPlugin',
            'en',
        )  # noqa: E501

        add_plugin(
            placeholder,
            'EmailField',
            'en',
            name='email_2',
            required=True,
            target=form_plugin2,
        )

        add_plugin(
            placeholder,
            'SubmitButton',
            'en',
            target=form_plugin2,
            label='Submit',
        )

        form_plugin2.action_backend = 'default'
        form_plugin2.save()

        page.publish('en')
        self.reload_urls()
        self.apphook_clear()

        response = self.client.post(page.get_absolute_url('en'), {
            'form_plugin_id': form_plugin2.id,
            'email_2': 'test@test',
        })

        email_field = '<input type="email" name="{name}"'
        self.assertContains(response, email_field.format(name='email_1'))
        self.assertContains(response, email_field.format(name='email_2'))



from email.utils import formataddr
from functools import partial

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _

from djangocms_text_ckeditor.fields import HTMLField
from emailit.api import construct_mail

from aldryn_forms.helpers import get_user_name
from aldryn_forms.models import FormPlugin

from .helpers import get_email_template_name, get_theme_template_name, render_text


EMAIL_THEMES = getattr(
    settings,
    "ALDRYN_FORMS_EMAIL_THEMES",
    [('default', _('default'))]
)


class EmailNotificationFormPlugin(FormPlugin):
    class Meta:
        proxy = True

    def copy_relations(self, oldinstance):
        for item in oldinstance.email_notifications.all():
            item.pk = None
            item.form = self
            item.save()

    def get_notification_conf(self):
        plugin_class = self.get_plugin_class()
        return plugin_class.notification_conf_class(form_plugin=self)

    def get_notification_text_context(self, form):
        notification_conf = self.get_notification_conf()
        text_context = notification_conf.get_context(form)
        return text_context

    def get_notification_text_context_keys_as_choices(self):
        notification_conf = self.get_notification_conf()
        choices = notification_conf.get_context_keys_as_choices()
        return choices


class EmailNotification(models.Model):

    class Meta:
        verbose_name = _('Email notification')
        verbose_name_plural = _('Email notifications')

    theme = models.CharField(
        verbose_name=_('theme'),
        max_length=200,
        help_text=_('Provides the base theme for the email.'),
        choices=EMAIL_THEMES
    )
    to_name = models.CharField(
        verbose_name=_('to name'),
        max_length=200,
        blank=True
    )
    to_email = models.CharField(
        verbose_name=_('to email'),
        max_length=200,
        blank=True
    )
    to_user = models.ForeignKey(
        to=getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
        verbose_name=_('to user'),
        blank=True,
        null=True,
        limit_choices_to={'is_staff': True},
        on_delete=models.CASCADE,
    )
    from_name = models.CharField(
        verbose_name=_('from name'),
        max_length=200,
        blank=True
    )
    from_email = models.CharField(
        verbose_name=_('from email'),
        max_length=200,
        blank=True
    )
    reply_to_email = models.CharField(
        verbose_name=_('reply to email'),
        max_length=200,
        blank=True
    )
    subject = models.CharField(
        verbose_name=_("subject"),
        max_length=200,
        blank=True
    )
    body_text = models.TextField(
        verbose_name=_('email body (txt)'),
        blank=True,
        help_text=_('used when rendering the email in text only mode.')
    )
    body_html = HTMLField(
        verbose_name=_('email body (html)'),
        blank=True,
        help_text=_('used when rendering the email in html.')
    )
    form = models.ForeignKey(
        to=EmailNotificationFormPlugin,
        related_name='email_notifications',
        on_delete=models.CASCADE,
    )

    def __str__(self):
        to_name = self.get_recipient_name()
        to_email = self.get_recipient_email()
        return u'{0} ({1})'.format(to_name, to_email)

    def clean(self):
        recipient_email = self.get_recipient_email()

        if self.pk and not recipient_email:
            message = gettext('Please provide a recipient.')
            raise ValidationError(message)

    def get_recipient_name(self):
        if self.to_name:
            # manual name takes precedence over user relationship.
            name = self.to_name
        elif self.to_user_id:
            name = get_user_name(self.to_user)
        else:
            name = ''
        return name

    def get_recipient_email(self):
        if self.to_email:
            # manual email takes precedence over user relationship.
            email = self.to_email
        elif self.to_user_id:
            email = self.to_user.email
        else:
            email = ''
        return email

    def get_email_context(self, form):
        get_template = partial(get_theme_template_name, theme=self.theme)

        context = {
            'form_plugin': self.form,
            'form_data': form.get_serialized_field_choices(is_confirmation=True),
            'form_name': self.form.name,
            'email_notification': self,
            'email_html_theme': get_template(suffix='html'),
            'email_txt_theme': get_template(suffix='txt'),
        }
        return context

    def get_email_kwargs(self, form):
        form_plugin = self.form

        text_context = form_plugin.get_notification_text_context(form)

        email_context = self.get_email_context(form)
        email_context['text_context'] = text_context

        notification_conf = form_plugin.get_notification_conf()

        kwargs = {
            'context': email_context,
            'language': form_plugin.language,
            # needs to be empty string because emailit expects a string
            # it's empty so the template lookup fails.
            'template_base': '',
            'body_templates': [
                notification_conf.get_txt_email_template_name()],
            'subject_templates': [
                get_email_template_name(name='subject', suffix='txt')],
        }

        if notification_conf.html_email_format_enabled:
            # we only want to render html template if html format is enabled.
            kwargs['html_templates'] = [
                notification_conf.get_html_email_template_name()]

        render = partial(render_text, context=text_context)

        recipient_name = self.get_recipient_name()

        recipient_email = self.get_recipient_email()
        recipient_email = render(recipient_email)

        if recipient_name:
            recipient_name = render(recipient_name)
            recipient_email = formataddr((recipient_name, recipient_email))

        kwargs['recipients'] = [recipient_email]

        if self.from_email:
            from_email = render(self.from_email)

            if self.from_name:
                from_name = render(self.from_name)
                from_email = formataddr((from_name, from_email))

            kwargs['from_email'] = from_email

        if self.reply_to_email:
            reply_to_email_rendered = render(self.reply_to_email)
            kwargs['reply_to'] = [reply_to_email_rendered]

        return kwargs

    def prepare_email(self, form):
        email_kwargs = self.get_email_kwargs(form)
        return construct_mail(**email_kwargs)

    def render_body_text(self, context):
        return render_text(self.body_text, context)

    def render_body_html(self, context):
        return render_text(self.body_html, context)

    def render_subject(self, context):
        return render_text(self.subject, context)



from django.contrib.auth.models import User
from django.core import mail

from cms.api import add_plugin, create_page
from cms.test_utils.testcases import CMSTestCase

from tests.test_views import CMS_3_6

from aldryn_forms.models import FormSubmission


class FormPluginTestCase(CMSTestCase):
    def setUp(self):
        super().setUp()

        self.page = create_page('test page', 'test_page.html', 'en', published=True)
        try:
            self.placeholder = self.page.placeholders.get(slot='content')
        except AttributeError:
            self.placeholder = self.page.get_placeholders('en').get(slot='content')
        self.user = User.objects.create_superuser('username', 'email@example.com', 'password')

        plugin_data = {
            'redirect_type': 'redirect_to_url',
            'url': 'http://www.google.com',
        }
        self.form_plugin = add_plugin(self.placeholder, 'FormPlugin', 'en', **plugin_data)
        self.form_plugin.recipients.add(self.user)

        add_plugin(self.placeholder, 'SubmitButton', 'en', target=self.form_plugin)

    def test_form_submission_default_action(self):
        self.form_plugin.action_backend = 'default'
        self.form_plugin.save()
        if CMS_3_6:
            self.page.publish('en')

        response = self.client.post(self.page.get_absolute_url('en'), {})

        self.assertEqual(response.status_code, 200)
        self.assertEqual(FormSubmission.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_form_submission_email_action(self):
        self.form_plugin.action_backend = 'email_only'
        self.form_plugin.save()
        if CMS_3_6:
            self.page.publish('en')

        response = self.client.post(self.page.get_absolute_url('en'), {})

        self.assertEqual(response.status_code, 200)
        self.assertEqual(FormSubmission.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_form_submission_no_action(self):
        self.form_plugin.action_backend = 'none'
        self.form_plugin.save()
        if CMS_3_6:
            self.page.publish('en')

        response = self.client.post(self.page.get_absolute_url('en'), {})

        self.assertEqual(response.status_code, 200)
        self.assertEqual(FormSubmission.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)


class EmailNotificationFormPluginTestCase(CMSTestCase):
    def setUp(self):
        super().setUp()

        self.page = create_page('test page', 'test_page.html', 'en', published=True)
        try:
            self.placeholder = self.page.placeholders.get(slot='content')
        except AttributeError:
            self.placeholder = self.page.get_placeholders('en').get(slot='content')
        self.user = User.objects.create_superuser('username', 'email@example.com', 'password')

        plugin_data = {
            'redirect_type': 'redirect_to_url',
            'url': 'http://www.google.com',
        }
        self.form_plugin = add_plugin(self.placeholder, 'EmailNotificationForm', 'en', **plugin_data)
        self.form_plugin.email_notifications.create(to_user=self.user, theme='default')

        add_plugin(self.placeholder, 'SubmitButton', 'en', target=self.form_plugin)

    def test_form_submission_default_action(self):
        self.form_plugin.action_backend = 'default'
        self.form_plugin.save()
        if CMS_3_6:
            self.page.publish('en')

        response = self.client.post(self.page.get_absolute_url('en'), {})

        self.assertEqual(response.status_code, 200)
        self.assertEqual(FormSubmission.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_form_submission_email_action(self):
        self.form_plugin.action_backend = 'email_only'
        self.form_plugin.save()
        if CMS_3_6:
            self.page.publish('en')

        response = self.client.post(self.page.get_absolute_url('en'), {})

        self.assertEqual(response.status_code, 200)
        self.assertEqual(FormSubmission.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_form_submission_no_action(self):
        self.form_plugin.action_backend = 'none'
        self.form_plugin.save()
        if CMS_3_6:
            self.page.publish('en')

        response = self.client.post(self.page.get_absolute_url('en'), {})

        self.assertEqual(response.status_code, 200)
        self.assertEqual(FormSubmission.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)



from django.utils.translation import gettext_lazy as _

from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool


class FormsApp(CMSApp):
    name = _('Forms')

    def get_urls(self, *args, **kwargs):
        return ['aldryn_forms.urls']


apphook_pool.register(FormsApp)



from django.http import HttpResponseBadRequest, HttpResponseRedirect
from django.shortcuts import render
from django.urls import resolve

from .models import FormPlugin
from .utils import get_plugin_tree


try:
    from cms.utils.page import get_page_from_request
except ImportError:
    # for django-cms<3.5
    from cms.utils.page_resolver import get_page_from_request


def submit_form_view(request):
    cms_page = get_page_from_request(request)

    if not cms_page:
        return HttpResponseBadRequest()

    template = cms_page.get_template()

    context = {
        'current_app': resolve(request.path).namespace,
        'current_page': cms_page,
    }

    if request.method == 'POST':
        form_plugin_id = request.POST.get('form_plugin_id') or ''

        if not form_plugin_id.isdigit():
            # fail if plugin_id has been tampered with
            return HttpResponseBadRequest()

        try:
            # I believe this could be an issue as we don't check if the form submitted
            # is in anyway tied to this page.
            # But then we have a problem with static placeholders :(
            form_plugin = get_plugin_tree(FormPlugin, pk=form_plugin_id)
        except FormPlugin.DoesNotExist:
            return HttpResponseBadRequest()

        form_plugin_instance = form_plugin.get_plugin_instance()[1]
        # saves the form if it's valid
        form = form_plugin_instance.process_form(form_plugin, request)
        success_url = form_plugin_instance.get_success_url(instance=form_plugin)

        if form.is_valid() and success_url:
            return HttpResponseRedirect(success_url)
    return render(request, template, context)



# Generated by Django 2.0 on 2019-01-04 11:42

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('email_notifications', '0003_auto_20180206_1733'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailnotification',
            name='to_user',
            field=models.ForeignKey(blank=True, limit_choices_to={'is_staff': True}, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='to user'),
        ),
    ]



DEFAULT_ALDRYN_FORMS_ACTION_BACKENDS = {
    'default': 'aldryn_forms.action_backends.DefaultAction',
    'email_only': 'aldryn_forms.action_backends.EmailAction',
    'none': 'aldryn_forms.action_backends.NoAction',
}
ALDRYN_FORMS_ACTION_BACKEND_KEY_MAX_SIZE = 15



import abc


class BaseAction(metaclass=abc.ABCMeta):

    @abc.abstractproperty
    def verbose_name(self):
        pass  # pragma: no cover

    @abc.abstractmethod
    def form_valid(self, cmsplugin, instance, request, form):
        pass  # pragma: no cover



import logging
from email.utils import parseaddr
from typing import List

from django.contrib import admin
from django.core.mail import get_connection
from django.template.defaultfilters import safe
from django.utils.translation import gettext_lazy as _

from cms.plugin_pool import plugin_pool

from aldryn_forms.cms_plugins import FormPlugin
from aldryn_forms.validators import is_valid_recipient

from .models import EmailNotification, EmailNotificationFormPlugin
from .notification import DefaultNotificationConf


logger = logging.getLogger(__name__)


class NewEmailNotificationInline(admin.StackedInline):
    extra = 1
    fields = ['theme']
    model = EmailNotification
    verbose_name = _('new email notification')
    verbose_name_plural = _('new email notifications')

    fieldsets = (
        (None, {
            'fields': (
                'theme',
            )
        }),
    )

    def get_queryset(self, request):
        queryset = super(NewEmailNotificationInline, self).get_queryset(request)
        return queryset.none()


class ExistingEmailNotificationInline(admin.StackedInline):
    model = EmailNotification

    fieldsets = (
        (None, {
            'fields': (
                'theme',
            )
        }),
        (_('Recipients'), {
            'classes': ('collapse',),
            'fields': (
                'text_variables',
                'to_user',
                ('to_name', 'to_email'),
                ('from_name', 'from_email'),
                'reply_to_email',
            )
        }),
    )

    readonly_fields = ['text_variables']

    text_variables_help_text = _(
        'the variables can be used within the email body, email sender,'
        'and other notification fields'
    )

    def has_add_permission(self, request, obj=None):
        return False

    def get_fieldsets(self, request, obj=None):
        fieldsets = super(ExistingEmailNotificationInline, self).get_fieldsets(request, obj)

        if obj is None:
            return fieldsets

        email_fieldset = self.get_email_fieldset(obj)
        fieldsets = list(fieldsets) + email_fieldset
        return fieldsets

    def get_email_fieldset(self, obj):
        fields = ['subject']

        notification_conf = obj.get_notification_conf()

        if notification_conf.txt_email_format_configurable:
            # add the body_text field only if it's configurable
            fields.append('body_text')

        if notification_conf.html_email_format_enabled:
            # add the body_html field only if email is allowed
            # to be sent in html version.
            fields.append('body_html')
        return [(_('Email'), {
            'classes': ('collapse',),
            'fields': fields
        })]

    def text_variables(self, obj: EmailNotification) -> str:
        if obj.pk is None:
            return ''

        # list of tuples - [('category', [('value', 'label')])]
        choices_by_category = obj.form.get_notification_text_context_keys_as_choices()

        var_items: List[str] = []
        for category, choices in choices_by_category:
            for choice_tuple in choices:
                field_value = choice_tuple[0]
                var_items += '<li>${' + field_value + '}</li>'

        vars_html_list = f'<p>{"".join(var_items)}</p>'
        help_text = (
            f'<p class="help">'
            f'{_("the variables can be used within the email body, email sender, and other notification fields")}'
            f'</p>'
        )
        return safe(vars_html_list + u'\n' + help_text)
    text_variables.allow_tags = True
    text_variables.short_description = _('available text variables')

    class Media:
        css = {
            'all': ['email_notifications/admin/email-notifications.css']
        }


class EmailNotificationForm(FormPlugin):
    name = _('Form (Advanced)')
    model = EmailNotificationFormPlugin
    inlines = [
        ExistingEmailNotificationInline,
        NewEmailNotificationInline
    ]
    notification_conf_class = DefaultNotificationConf
    child_ckeditor_body_css_class = 'aldryn-forms-form-plugin'

    fieldsets = (
        (None, {
            'fields': (
                'name',
                'redirect_type',
                ('redirect_page', 'url'),
            )
        }),
        (_('Advanced Settings'), {
            'classes': ('collapse',),
            'fields': (
                'is_enable_autofill_from_url_params',
                'form_template',
                'error_message',
                'success_message',
                'custom_classes',
                'action_backend',
            )
        }),
    )

    def get_inline_instances(self, request, obj=None):
        inlines = super(EmailNotificationForm, self).get_inline_instances(request, obj)

        if obj is None:
            # remove ExistingEmailNotificationInline inline instance
            # if we're first creating this object.
            inlines = [inline for inline in inlines
                       if not isinstance(inline, ExistingEmailNotificationInline)]
        return inlines

    def send_notifications(self, instance, form):
        try:
            connection = get_connection(fail_silently=False)
            connection.open()
        except Exception as msg:
            # I use a "catch all" in order to not couple this handler to a specific email backend
            # different email backends have different exceptions.
            logger.exception(f"Could not send notification emails. {msg}")
            return []

        notifications = instance.email_notifications.select_related('form')

        emails = []
        recipients = []

        for notification in notifications:
            email = notification.prepare_email(form=form)

            to_email = email.to[0]

            if is_valid_recipient(to_email):
                emails.append(email)
                recipients.append(parseaddr(to_email))

        try:
            connection.send_messages(emails)
        except Exception as msg:
            # again, we catch all exceptions to be backend agnostic
            logger.exception(f"Could not send notification emails. {msg}")
            recipients = []
        return recipients


plugin_pool.register_plugin(EmailNotificationForm)



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0007_auto_20170315_1421'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailfieldplugin',
            name='max_value',
            field=models.PositiveIntegerField(null=True, verbose_name='Max value', blank=True),
        ),
        migrations.AlterField(
            model_name='fieldplugin',
            name='max_value',
            field=models.PositiveIntegerField(null=True, verbose_name='Max value', blank=True),
        ),
        migrations.AlterField(
            model_name='fileuploadfieldplugin',
            name='max_value',
            field=models.PositiveIntegerField(null=True, verbose_name='Max value', blank=True),
        ),
        migrations.AlterField(
            model_name='imageuploadfieldplugin',
            name='max_value',
            field=models.PositiveIntegerField(null=True, verbose_name='Max value', blank=True),
        ),
        migrations.AlterField(
            model_name='textareafieldplugin',
            name='max_value',
            field=models.PositiveIntegerField(null=True, verbose_name='Max value', blank=True),
        ),
    ]



# Generated by Django 4.2.3 on 2023-07-24 09:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("email_notifications", "0005_add_field_reply_to_email"),
    ]

    operations = [
        migrations.AlterField(
            model_name="emailnotification",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]



from django.contrib import admin

from tablib import Dataset

from ..models import FormSubmission
from .base import BaseFormSubmissionAdmin
from .views import FormExportWizardView


def get_supported_format():
    """Get supported format from types xlsx, xls, tsv or cvs."""
    dataset = Dataset()
    for ext in ('xlsx', 'xls'):
        try:
            getattr(dataset, ext)
            return ext
        except (ImportError, AttributeError):
            pass
    return 'csv'


class FormSubmissionAdmin(BaseFormSubmissionAdmin):
    readonly_fields = BaseFormSubmissionAdmin.readonly_fields + ['form_url']

    def get_form_export_view(self):
        return FormExportWizardView.as_view(admin=self, file_type=get_supported_format())


admin.site.register(FormSubmission, FormSubmissionAdmin)
